
clear all;
load weekdays_data.mat;

% We would like to forecast the day 101 using same days of the week data.
% We assume that day 101 is known up to 10:05 AM.
daytoForecast=101;
daytoForecastData=data(1:37,daytoForecast);
% Find the same days of the week data, the relevant days for the forecast.
weekdays=weekdays(1:101);
relevantDays=find(weekdays==weekdays(daytoForecast) );
relevantDays(length(relevantDays))=[]; % delete day 101
daysData=data(:,relevantDays);

% Use cross validation to find optimal parametrs for the continuation
[npca,lambda,var_coef]=cross_validation_continuation(daysData, 37,5);


% Perform the forecast
options.confidence_band=1;
options.number_pca=npca;
options.var_coef=var_coef;
options.lambda_ridge=lambda;

[forecasted_data,forecasted_fd,confidence_band]=...
    forecast_continuation(daysData, daytoForecastData,options);


% Plot the results, include confidence band

figure(1);clf;
set(gcf,'Position',[100,100,620,320]);%,'Color',[1,1,1]);
scatter(7:1/12:21,data(:,daytoForecast),24,'ok','filled');hold on
plot(10+1/12:1/12:21,forecasted_data,'-r','LineWidth',3);hold on
plot(10+1/12:1/12:21,confidence_band(:,1),'-b','LineWidth',3);hold on
plot(10+1/12:1/12:21,confidence_band(:,2),'-b','LineWidth',3);hold on
set(gca,'XTick',[7:3:21],'YTick',[],'LineWidth',2,'LineStyle','-','Clipping','off','XLim',[7 21], 'YLim',[0,400]);
set(gca,'YTick',[0:100:400]);
ylabel('Call Counts','fontsize',12,'fontweight','b')
title('Forecast & Confidence Band','FontSize',12,...
    'FontWeight','bold');hold on;