function b_spline = bspline_fd2bspline(bspline_fd)
%  BSPLINE_FD2BSPLINE convert an fd of bsplines function to an array of
%  spline in the usual B-spline rpersentation in Matlab 
%
%  Arguments:
%  bspline_fd - an fd object with Bspline basis
%  Return:
%  b_spline - an array of B-spline.

%  Last modified 12 March 2008

%  check bspline_fd

if ~strcmp(class(bspline_fd),'fd')
    error('BSPLINE_FD is not a functional data object.');
end

basis = getbasis(bspline_fd);
type  = getbasistype(basis);
if ~strcmp(type,'bspline')
    error('BSPLINE_FD does not have a B-spline basis.');
end
range  = getbasisrange(basis);
breaks = [range(1),getbasispar(basis),range(2)];
nbasis = getnbasis(basis);
norder = nbasis - length(breaks) + 2;

coef = getcoef(bspline_fd);
replications=size(coef,2);

b_spline=[];
for index=1:replications
   sp = spmak(augknt(breaks,norder),coef(:,index)');
   b_spline=[b_spline,sp];
end
