function [npca,lambda,var_coef]= ...
    cross_validation_continuation(data,break_index,nfolds,delta,options)
% CROSS_VALIDATION_CONTINUATION finds parmeters to use in the function
% FORECAST_CONTINUATION. The parmetrs are: number of pca components,
%the regularization parmeter and the variance coeficient for the confidence band.   
%  
% Arguments:
%     data: an pxn matrix where n is the number of past vectors and p is their length.
%     break_index: the end point of the beginning part of the curves
%     nfolds- number of folds for the cross-validation , default=5
%     delta - confidence level, default=0.05
%     options: a struct -
%       lambda_smoothnesss: the penalty parameter controlling the smoothness of
%                          the curves, default 1. 
%        norder: order of the splines, default- 4 (cubic)%  
%       pca_range : The number of pca components range for the CV
%                         default=1:6
%       lambda_range : the  lambda range for the CV
%                         default=0.1:0.1:1
%       var_coef_range: the var_coef range for the CV
%
%
%       Last modified: 16 Novmber 2009
%


if(~exist('options')|| ~isfield(options,'lambda_smoothnesss')) options.lambda_smoothnesss=1;end;
if(~isfield(options,'norder')) options.norder=4;end;
if(~isfield(options,'pca_range')) pca_range=1;
else pca_range=options.pca_range;end;
if(~isfield(options,'lambda_range')) lambda_range=0.1:0.3:1;
else lambda_range=options.lambda_range;end;
if(~isfield(options,'var_coef_range')) var_coef_range=[2,6];
else  var_coef_range=options.var_coef_range;end;

if(nargin<4) delta=0.05;end
if(nargin<3) nfolds=5;end

%create folds
n=size(data,2);
perm=randperm(n);
modn=mod(1:n,nfolds)+1;

fold_indices=cell(nfolds,1);
not_in_fold_indices=cell(nfolds,1);

pfolds=[];
for(f=1:nfolds)
    fold_indices{f}=perm(modn==f);
     pfolds=[pfolds,fold_indices{f}];
    not_in_fold_indices{f}=perm(modn~=f);
end


med_rmse=zeros(length(pca_range),length(lambda_range));
for(i=1:length(pca_range))
    npca=pca_range(i);
    for(j=1:length(lambda_range))
        lambda=lambda_range(j);
        rmse_scores=[];
        for(f=1:nfolds)
            options.number_pca=npca;
            options.lambda_ridge=lambda;
            forecasted_data=forecast_continuation(data(:, not_in_fold_indices{f}),...
                data(1:break_index, fold_indices{f}),options);
            rmse_scores=[rmse_scores,...
                rmse(data(break_index+1:end, fold_indices{f}),forecasted_data)];
            
            
        end
        med_rmse(i,j)=median(rmse_scores);
    end
end
min_val=min(min(med_rmse));
[i,j]=ind2sub(size(med_rmse),find(med_rmse==min_val,1));
npca=pca_range(i);
lambda=lambda_range(j);

forecasted_data=[];D=[];options.confidence_band=0;

for(f=1:nfolds)
    options.number_pca=npca;
    options.lambda_ridge=lambda;
    options.var_coef=1;
    options.confidence_band=1;
    [forecasted,forecasted_fd,confidence_band]=forecast_continuation(data(:, not_in_fold_indices{f}),...
        data(1:break_index, fold_indices{f}),options);
    forecasted_data=[forecasted_data,forecasted];
    D=[D,forecasted-confidence_band(:,1:size(forecasted,2))];
end
var_coef=fminbnd(@(x) abs(delta-cross_band(data(break_index+1:end,pfolds),forecasted_data,x, D)),...
    var_coef_range(1),var_coef_range(2));



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function delta= cross_band(data,forecast,c, D )

delta=sum(sum((data-(forecast+c*D)>0 | data-(forecast-c*D)<0)))/numel(data);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [rmse_vec,rmse_avg] = rmse(original,forecasted)

[n,p]=size(original);
rmse_vec=(sum((original-forecasted).^2,1)/n).^0.5;
rmse_avg=sum(rmse_vec)/p;
