function restrictedfd = restrict_bsplines_fd(bspline_fd,left_k,right_k)
%  RESTRICTED_BSPLINE_FD computes the functional data obtained from
%  bspline_fd which define on the segment [a,b]  by restriction to the
%  segment to the knots. It is assumed that a<= left_k<right_k<=b.
%  
%  Arguments:
%  bspline_fd - an fd object with B-spline basis on some segment [a,b].
%  [left_k,right_k] - the segment in which the bspline_fd is restricted to.
%
%  Return:
%  restrictedfd - the restricted fd obj to the segment [left_k,right-k]

%  Last modified 7 July 2009

%  check bspline_fd

if ~strcmp(class(bspline_fd),'fd')
    error('BSPLINE_FD is not a functional data object.');
end

basis = getbasis(bspline_fd);
type  = getbasistype(basis);
if ~strcmp(type,'bspline')
    error('BSPLINE_FD does not have a B-spline basis.');
end
range  = getbasisrange(basis);
breaks = [range(1),getbasispar(basis),range(2)];
nbasis = getnbasis(basis);
norder = nbasis - length(breaks) + 2;


if  (left_k < range(1) || right_k > range(2))
    error('The segment [LEFT_K,RIGHT_K  is not in the range of BSPLINE_FD.');
end


coef = getcoef(bspline_fd);
replications=size(coef,2);

newcoef=[];
for index=1:replications
   sp = spmak(augknt(breaks,norder),coef(:,index)');
   restricted_sp = fnbrk(sp,[left_k,right_k]);
   newcoef=[newcoef,restricted_sp.coefs'];
end
restricted_basis=create_bspline_basis([left_k,right_k],restricted_sp.number,norder,knt2brk(restricted_sp.knots));
restrictedfd=fd(newcoef, restricted_basis, getnames(bspline_fd));